<?php

declare(strict_types=1);

namespace mirzaev\site\account\models;

// Фреймворк PHP
use mirzaev\minimal\model;

// Фреймворк ArangoDB
use mirzaev\arangodb\connection as arangodb;

// Встроенные библиотеки
use exception,
	redis,
	redisexception;

/**
 * Ядро моделей
 *
 * @package mirzaev\site\account\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class core extends model
{
	/**
	 * Постфикс
	 */
	final public const POSTFIX = '';

	/**
	 * Путь до файла с настройками подключения к базе данных ArangoDB
	 */
	final public const ARANGODB = '../settings/arangodb.php';

	/**
	 * Путь до файла с настройками подключения к базе данных Redis
	 */
	final public const REDIS = '../settings/redis.php';

	/**
	 * Соединение с базой данных ArangoDB
	 */
	protected static arangodb $arangodb;

	/**
	 * Соединение с базой данных Redis 
	 */
	protected static redis $redis;

	/**
	 * Конструктор
	 *
	 * @param bool $initialize Инициализировать контроллер?
	 * @param ?arangodb $arangodb Инстанция соединения с базой данных ArangoDB
	 * @param ?redis $redis Инстанция соединения с базой данных Redis
	 */
	public function __construct(bool $initialize = true, ?arangodb $arangodb = null, ?redis $redis = null)
	{
		parent::__construct($initialize);

		if ($initialize) {
			// Запрошена инициализация

			if (isset($arangodb)) {
				// Получена инстанция соединения с базой данных

				// Запись и инициализация соединения с базой данных
				$this->__set('arangodb', $arangodb);
			} else {
				// Не получена инстанция соединения с базой данных

				// Инициализация соединения с базой данных по умолчанию
				$this->__get('arangodb');
			}

			if (isset($redis)) {
				// Получена инстанция соединения с базой данных

				// Запись и инициализация соединения с базой данных
				$this->__set('redis', $redis);
			} else {
				// Не получена инстанция соединения с базой данных

				// Инициализация соединения с базой данных по умолчанию
				$this->__get('redis');
			}
		}
	}

	/**
	 * Записать свойство
	 *
	 * @param string $name Название
	 * @param mixed $value Значение
	 */
	public function __set(string $name, mixed $value = null): void
	{
		match ($name) {
			'arangodb' => (function () use ($value) {
				if ($this->__isset('arangodb')) {
					// Свойство уже было инициализировано

					// Выброс исключения (неудача)
					throw new exception('Запрещено реинициализировать соединение с базой данных ArangoDB ($this::$arangodb)', 500);
				} else {
					// Свойство ещё не было инициализировано

					if ($value instanceof arangodb) {
						// Передано подходящее значение

						// Запись свойства (успех)
						self::$arangodb = $value;
					} else {
						// Передано неподходящее значение

						// Выброс исключения (неудача)
						throw new exception('Соединение с базой данных ArangoDB ($this::$arangodb) должно быть инстанцией mirzaev\arangodb\connection', 500);
					}
				}
			})(),
			'redis' => (function () use ($value) {
				if ($this->__isset('redis')) {
					// Свойство уже было инициализировано

					// Выброс исключения (неудача)
					throw new exception('Запрещено реинициализировать соединение с базой данных Redis ($this::$redis)', 500);
				} else {
					// Свойство ещё не было инициализировано

					if ($value instanceof redis) {
						// Передано подходящее значение

						// Запись свойства (успех)
						self::$redis = $value;
					} else {
						// Передано неподходящее значение

						// Выброс исключения (неудача)
						throw new exception('Соединение с базой данных Redis ($this::$arangodb) должно быть инстанцией redis', 500);
					}
				}
			})(),

			default => parent::__set($name, $value)
		};
	}

	/**
	 * Прочитать свойство
	 *
	 * @param string $name Название
	 *
	 * @return mixed Содержимое
	 */
	public function __get(string $name): mixed
	{
		return match ($name) {
			'arangodb' => (function () {
				try {
					if (!$this->__isset('arangodb')) {
						// Свойство не инициализировано

						// Инициализация значения по умолчанию исходя из настроек
						$this->__set('arangodb', new arangodb(require static::ARANGODB));
					}

					return self::$arangodb;
				} catch (exception) {
					return null;
				}
			})(),
			'redis' => (function () {
				try {
					if (!$this->__isset('redis')) {
						// Свойство не инициализировано

						// Инициализация настроек
						[$connect, $authentication] = require static::REDIS;

						// Инициализация инстанции redis
						$redis = new redis;

						// Подключение к базе данных redis
						$redis->pconnect(...$connect);

						// Аутентификация
						$redis->auth($authentication);

						// Выбор базы данных
						$redis->select(1);

						// Инициализация значения по умолчанию исходя из настроек
						$this->__set('redis', $redis);
					}

					return self::$redis;
				} catch (redisexception) {
					return null;
				}
			})(),
			default => parent::__get($name)
		};
	}

	/**
	 * Проверить свойство на инициализированность
	 *
	 * @param string $name Название
	 */
	public function __isset(string $name): bool
	{
		return match ($name) {
			default => parent::__isset($name)
		};
	}

	/**
	 * Удалить свойство
	 *
	 * @param string $name Название
	 */
	public function __unset(string $name): void
	{
		match ($name) {
			default => parent::__isset($name)
		};
	}


	/**
	 * Статический вызов
	 *
	 * @param string $name Название
	 * @param array $arguments Параметры
	 */
	public static function __callStatic(string $name, array $arguments): mixed
	{
		match ($name) {
			'arangodb' => (new static)->__get('arangodb'),
			'redis' => (new static)->__get('redis'),
			default => throw new exception("Не найдено свойство или функция: $name", 500)
		};
	}
}
